#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const d3 = require("d3");
const fs = require("fs");
const os = require("os");
const path = require("path");
const yargs = require("yargs");
const evolution_1 = require("./evolution");
const comparisonz_1 = require("./comparisonz");
const jsdom_1 = require("jsdom");
// TODO At each branch choose between defined type specifications with type
// guards
// Curves
const curves = {
    "linear": d3.curveLinear,
    "step": d3.curveStep,
    "step_before": d3.curveStepBefore,
    "step_after": d3.curveStepAfter,
    "basis": d3.curveBasis,
    "cardinal": d3.curveCardinal,
    "monotone": d3.curveMonotoneX,
    "catmull_rom": d3.curveCatmullRom,
};
// Symbols
const symbols = {
    "circle": d3.symbolCircle,
    "cross": d3.symbolCross,
    "diamond": d3.symbolDiamond,
    "square": d3.symbolSquare,
    "star": d3.symbolStar,
    "triangle": d3.symbolTriangle,
    "wye": d3.symbolWye,
};
// Symbols
const scales = {
    "linear": d3.scaleLinear,
    "log": d3.scaleLog,
    "sqrt": d3.scaleSqrt,
};
// Dict of plotting evolution types
const evoDatum = {
    "line": (plot, data) => plot.line(data),
    "span": (plot, data) => plot.span(data),
};
// Rendering functions
function evo(config, svg) {
    config.xaxis = config.xaxis || {};
    config.yaxis = config.yaxis || {};
    config.xaxis.format = d3.format(config.xaxis.format || "");
    config.yaxis.format = d3.format(config.yaxis.format || "");
    config.xaxis.scale = scales[config.xaxis.scale || "linear"]();
    config.yaxis.scale = scales[config.yaxis.scale || "linear"]();
    const plot = new evolution_1.Evolution(config.width || 162, config.height || 100)
        .xaxis(config.xaxis).yaxis(config.yaxis);
    config.xaxis.min === undefined || plot.xmin(config.xaxis.min);
    config.xaxis.max === undefined || plot.xmax(config.xaxis.max);
    config.yaxis.min === undefined || plot.ymin(config.yaxis.min);
    config.yaxis.max === undefined || plot.ymax(config.yaxis.max);
    config.classed === undefined || plot.classed(config.classed);
    config.data.forEach((datum) => evoDatum[datum.type](plot, datum.data)
        .label(datum.label || "")
        .classed(datum.classed || "")
        .curve(curves[datum.curve || "catmull_rom"])
        .point(d3.symbol().size(1).type(symbols[datum.point || "circle"])));
    plot.plot(svg);
}
function compz(config, svg) {
    config.axis = config.axis || {};
    config.axis.format = d3.format(config.axis.format || "");
    const plot = new comparisonz_1.ComparisonZ(config.width || 162)
        .numberFormat(d3.format(config.format || ""))
        .axis(config.axis);
    config.axis.max === undefined || plot.max(config.axis.max);
    config.classed === undefined || plot.classed(config.classed);
    config.data.forEach((datum) => {
        const { name, value } = datum;
        plot.value(name, value).classed(datum.classed || "");
    });
    plot.plot(svg);
}
const funcs = {
    "evolution": evo,
    "comparisonz": compz,
};
const styleTexts = [];
const stylePaths = [
    path.join(path.dirname(__dirname), "resources", "princ_style.css"),
    path.join(os.homedir(), ".princ_style.css"),
    ".princ_style.css",
];
const args = yargs
    .option("input", {
    "alias": "i",
    "default": "stdin",
    "describe": "Take input json from file.",
})
    .option("output", {
    "alias": "o",
    "default": "stdout",
    "describe": "Output svg to file.",
})
    .option("style", {
    "alias": ["css", "c", "s"],
    "array": true,
    "default": [],
    "describe": "Add files as css styling for svg. Any file specified here will be added in addition to any of the following files in order: " + stylePaths.map(p => '"' + p + '"').join(", "),
})
    .help()
    .alias("version", "V")
    .alias("help", "h")
    .argv;
const config = JSON.parse(fs.readFileSync(args.input === "stdin" ? 0 : args.input, { encoding: "utf-8" }));
const dom = new jsdom_1.JSDOM();
const { window } = dom;
const { document } = window;
const svg = document.createElementNS("http://www.w3.org/2000/svg", "svg");
stylePaths.forEach((cssFile) => {
    try {
        styleTexts.push(fs.readFileSync(cssFile, { encoding: "utf-8" }));
    }
    catch (err) {
        if (err.code !== "ENOENT") {
            throw err;
        }
    }
});
args.style.forEach((cssFile) => {
    styleTexts.push(fs.readFileSync(cssFile, { encoding: "utf-8" }));
});
funcs[config.type](config, svg);
const style = document.createElement("style");
style.textContent = styleTexts.map(t => t.replace(/\s+/g, " ")).join(" ");
svg.appendChild(style);
// FIXME Make this better. This is necessary otherwise we get errors on output
const size = 4096;
let result = svg.outerHTML;
if (args.output === "stdout") {
    args.output = 1;
}
else {
    try {
        fs.truncateSync(args.output);
    }
    catch (err) {
        if (err.code !== "ENOENT") {
            throw err;
        }
    }
}
while (result.length > size) {
    fs.appendFileSync(args.output, result.substr(0, size));
    result = result.substr(size);
}
fs.appendFileSync(args.output, result);
